###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.5 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import MessageDialog
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import Products
import OFS.SimpleItem
import OFS.content_types
import cStringIO
import mimetools
import base_item
import string

rfc822_attachment_add_redirect = HTMLFile('rfc822_attachment_add_redirect',
					globals())
manage_add_rfc822_attachment_form = HTMLFile('rfc822_attachment_add',
						globals())

def manage_add_rfc822_attachment(self, file=None, data='', headers='',
					filename='', content_type='',
					RESPONSE=None):
	"Add an instance of the rfc822_attachment class"

	new_id = str(self.get_unique_id())

	# If we've got the RESPONSE object, this attachment was
	# submitted via the web.

	if RESPONSE is not None:

		data = file.read()
		filename = file.filename

		backslash_index = string.rfind(filename, '\\')

		# Now, this screws up unix filenames with backslahes
		# in them.. *ponder*

		if backslash_index > -1:

			filename = filename[backslash_index+1:]
			
		headers = {}
		content_type = OFS.content_types.guess_content_type(
				name=filename, body=data,
				default='application/octet-stream')[0]


	self._setObject(new_id,
		rfc822_attachment(new_id,
			data=data, content_type=content_type,
			filename=filename, headers=headers
		)
	)

	if RESPONSE is not None:

		RESPONSE.redirect(self.absolute_url())


class rfc822_attachment(
	Products.ZCatalog.CatalogAwareness.CatalogAware,
	OFS.SimpleItem.Item,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	base_item.base_item
	):
	"Documentation string"

	meta_type = 'RFC822Attachment'
	icon = 'misc_/Transform/dtmldoc.gif'

	meta_types = ()

	_properties = (
		{'id':'data', 'type':'text'},
		{'id':'content_type', 'type':'string'},
		{'id':'filename', 'type':'string'},
		)

	manage_options = (
		{'label':'Edit', 'action':'manage_main'},
		{'label':'View', 'action':''},
		)

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('View Attachment',
		 ('','render_data')),
		)

	index_html = HTMLFile('rfc822_attachment_index', globals())
	manage_main = HTMLFile('rfc822_attachment_edit', globals())


	def __init__(self, id, data='', content_type='', filename='',
			headers={}):
		"Initialize the class instance"

		self.id = id
		self.data = data
		self.size = len(data)
		self.content_type = content_type
		self.filename = filename
		self.headers = {}

		self.index_object()


	def encode(self):
		"""Returns encoded data."""

		out_file = cStringIO.StringIO()
		in_file = cStringIO.StringIO(self.data)
		in_file.seek(0)

		mimetools.encode(in_file, out_file, 'base64')

		return out_file.getvalue()


	def download(self, REQUEST=None, RESPONSE=None):
		"""Method for downloading / viewing attachment."""

		RESPONSE.setHeader('Content-Type', '%s; filename=%s' % \
					(self.content_type, self.filename))
		RESPONSE.setHeader('Content-Length', self.size)
		RESPONSE.setHeader('Content-Disposition', 'filename=%s' % self.filename)

		RESPONSE.write(self.data)


	def set_header(self, header_name, header_value):
		"""Sets an attachment header."""

		self.headers[header_name] = header_value


	def render_size(self, details=0):
		"""Returns a human-readable display of the size."""

		return_string = ""

		if self.size > 1024*1024:

			return_string = return_string + \
				"%.2f MB" % (self.size / 1024.0 / 1024)

		else:

			return_string = return_string + \
				"%.2f KB" % (self.size / 1024.0)

		if details:

			return_string = return_string + " (%s bytes)" % self.size

		return return_string


default__class_init__(rfc822_attachment)

